@@ -486,61 +486,117 @@ int dbd_st_internal_execute(SV* h, SV* statement, SV* attribs, int numParams,
 	&&  tolower(sbuf[0]) == 'l'
 	&&  tolower(sbuf[1]) == 'i'
 	&&  tolower(sbuf[2]) == 's'
-	&&  tolower(sbuf[3]) == 't'
-	&&  tolower(sbuf[4]) == 'f'
-	&&  tolower(sbuf[5]) == 'i'
-	&&  tolower(sbuf[6]) == 'e'
-	&&  tolower(sbuf[7]) == 'l'
-	&&  tolower(sbuf[8]) == 'd'
-	&&  tolower(sbuf[9]) == 's') {
-	char* table;
+	&&  tolower(sbuf[3]) == 't') {
+        if (slen >= 11
+	    &&  tolower(sbuf[4]) == 'f'
+	    &&  tolower(sbuf[5]) == 'i'
+	    &&  tolower(sbuf[6]) == 'e'
+	    &&  tolower(sbuf[7]) == 'l'
+	    &&  tolower(sbuf[8]) == 'd'
+	    &&  tolower(sbuf[9]) == 's'
+	    &&  isspace(sbuf[10])) {
+	    char* table;
 
-	while (slen && !isspace(*sbuf)) { --slen;  ++sbuf; }
-	while (slen && isspace(*sbuf)) { --slen;  ++sbuf; }
+	    slen -= 10;
+	    sbuf += 10;
+	    while (slen && isspace(*sbuf)) { --slen;  ++sbuf; }
 
-	if (!slen) {
-	    do_error(h, JW_ERR_QUERY, "Missing table name");
-	    return -2;
-	}
+	    if (!slen) {
+	        do_error(h, JW_ERR_QUERY, "Missing table name");
+		return -2;
+	    }
 
-	if (!(table = malloc(slen+1))) {
-	    do_error(h, JW_ERR_MEM, "Out of memory");
-	    return -2;
-	}
-	strncpy(table, sbuf, slen);
-	table[slen] = '\0';
-	*cdaPtr = MyListFields(svsock, sbuf);
-	free(table);
+	    if (!(table = malloc(slen+1))) {
+	        do_error(h, JW_ERR_MEM, "Out of memory");
+		return -2;
+	    }
+	    strncpy(table, sbuf, slen);
+	    sbuf = table;
+	    while (slen && !isspace(*sbuf)) { --slen;  ++sbuf; }
+	    *sbuf++ = '\0';
 
-	if (!(*cdaPtr)) {
-	    DO_ERROR(h, JW_ERR_LIST_FIELDS, svsock);
-	    return -2;
-	}
+	    *cdaPtr = MyListFields(svsock, table);
+	    free(table);
 
-	return 0;
-    } else {
-	if ((MyQuery(svsock, sbuf, slen) == -1)  &&
-	    (!MyReconnect(svsock, h)
-	     ||	 (MyQuery(svsock, sbuf, slen) == -1))) {
-	    Safefree(salloc);
-	    DO_ERROR(h, JW_ERR_QUERY, svsock);
-	    return -2;
-	}
-	Safefree(salloc);
+	    if (!(*cdaPtr)) {
+	        DO_ERROR(h, JW_ERR_LIST_FIELDS, svsock);
+		return -2;
+	    }
 
-	/** Store the result from the Query */
-#xtract Mysql
-	if (!(*cdaPtr = (use_mysql_use_result ?
-		 mysql_use_result(svsock) : mysql_store_result(svsock)))) {
-	    return mysql_affected_rows(svsock);
+	    return 0;
 #xtract Msql
-	if (!(*cdaPtr = MyStoreResult(svsock))) {
-	    return -1;
+	} else if (tolower(sbuf[4]) == 'i'
+		   &&  tolower(sbuf[5]) == 'n'
+		   &&  tolower(sbuf[6]) == 'd'
+		   &&  tolower(sbuf[7]) == 'e'
+		   &&  tolower(sbuf[8]) == 'x'
+		   &&  isspace(sbuf[9])) {
+	    char* table;
+	    char* index;
+
+	    slen -= 9;
+	    sbuf += 9;
+
+	    while (slen && isspace(*sbuf)) { --slen;  ++sbuf; }
+	    if (!slen) {
+	        do_error(h, JW_ERR_QUERY, "Missing table name");
+		return -2;
+	    }
+	    if (!(table = malloc(slen+1))) {
+	        do_error(h, JW_ERR_MEM, "Out of memory");
+		return -2;
+	    }
+	    strncpy(table, sbuf, slen);
+	    sbuf = table;
+
+	    while (slen && !isspace(*sbuf)) { --slen;  ++sbuf; }
+	    if (slen) {
+	        *sbuf++ = '\0';
+		--slen;
+	    }
+	    while (slen && isspace(*sbuf)) { --slen;  ++sbuf; }
+	    if (!slen) {
+	        do_error(h, JW_ERR_QUERY, "Missing index name");
+		free(table);
+		return -2;
+	    }
+	    index = sbuf;
+	    while (slen && !isspace(*sbuf)) { --slen;  ++sbuf; }
+	    *sbuf++ = '\0';
+
+	    *cdaPtr = msqlListIndex(svsock, table, index);
+	    free(table);
+	    if (!(*cdaPtr)) {
+	        DO_ERROR(h, JW_ERR_LIST_INDEX, svsock);
+		return -2;
+	    }
+
+	    return 0;
 #endxtract
 	}
-
-	return MyNumRows((*cdaPtr));
     }
+
+    if ((MyQuery(svsock, sbuf, slen) == -1)  &&
+	(!MyReconnect(svsock, h)
+	 ||	 (MyQuery(svsock, sbuf, slen) == -1))) {
+        Safefree(salloc);
+	DO_ERROR(h, JW_ERR_QUERY, svsock);
+	return -2;
+    }
+    Safefree(salloc);
+
+    /** Store the result from the Query */
+#xtract Mysql
+    if (!(*cdaPtr = (use_mysql_use_result ?
+		     mysql_use_result(svsock) : mysql_store_result(svsock)))) {
+        return mysql_affected_rows(svsock);
+#xtract Msql
+    if (!(*cdaPtr = MyStoreResult(svsock))) {
+        return -1;
+#endxtract
+    }
+
+    return MyNumRows((*cdaPtr));
 }
 
 
