@@ -16,7 +16,7 @@ use DynaLoader();
 use Carp ();
 @ISA = qw(DynaLoader);
 
-$VERSION = '2.01_01';
+$VERSION = '2.01_02';
 
 bootstrap DBD::mysql $VERSION;
 
@@ -188,24 +188,16 @@ sub ANSI2db {
     return $DBD::mysql::db::ANSI2db{"$type"};
 }
 
-sub listfields($$) {
-    my ($self, $table) = @_;
-    my ($sth);
-    if (!($sth = $self->prepare("LISTFIELDS $table"))) {
-	return undef;
-    }
-    if (!$sth->execute) {
-	return undef;
-    }
-    $sth;
-}
-
 sub _ListFields($$) {
     my($self, $table) = @_;
     if (!$DBD::mysql::QUIET) {
-	warn "'_ListFields' is deprecated, use 'listfields' instead.";
+	warn "'_ListFields' is deprecated, use the SQL query 'LISTFIELDS \$table' instead.";
     }
-    $self->func($table, 'listfields');
+    my($sth) = $self->prepare("LISTFIELDS $table");
+    if ($sth  &&  !$sth->execute()) {
+	undef $sth;
+    }
+    $sth;
 }
 
 sub admin {
@@ -261,10 +253,11 @@ Interface (DBI)
     @databases = DBD::mysql::dr->func( $hostname, '_ListDBs' );
     @tables = $dbh->func( '_ListTables' );
 
-    $sth = $dbh->listfields($table);
-    $sth->finish;
-
     $sth = $dbh->prepare("SELECT * FROM foo WHERE bla");
+       or
+    $sth = $dbh->prepare("LISTFIELDS $table");
+       or
+    $sth = $dbh->prepare("LISTINDEX $table $index");
     $sth->execute;
     $numRows = $sth->rows;
     $numFields = $sth->{'NUM_OF_FIELDS'};
@@ -281,7 +274,6 @@ Interface (DBI)
     $rc = $dbh->func('reload', 'admin');
 
 
-
 =head1 EXPERIMENTAL SOFTWARE
 
 This package contains experimental software and should *not* be used
@@ -380,30 +372,16 @@ an empty list is returned.
       }
 
 
-=item B<listfields>
-
-    $sth = $dbh->func($table, 'listfields');
-
-The listfields method returns a statement handle that can be used for
-obtaining informations about the columns of $table. For example you
-may consult $sth->{'type'} for the column types, $sth->{'NUM_OF_FIELDS'}
-for the number of columns and so on. See L</STATEMENT HANDLES> above.
-The method is equivalent to
-
-    $sth = $dbh->prepare("LISTFIELDS $table");
-    $sth->execute;
-
-In particular this means that you should ensure that $sth goes out of
-scope or call $sth->finish before disconnecting from the database, as
-you will receive a warning otherwise.
-
-
 =item B<ListFields>
 
 Deprecated, see L</COMPATIBILITY ALERT> below. Used to be equivalent
 to
 
-    $sth = $dbh->func($table, 'listfields');
+    $sth = $dbh->prepare("LISTFIELDS $table");
+    $sth->execute;
+
+See L</SQL EXTENSIONS> below.
+
 
 =item B<ListSelectedFields>
 
@@ -632,6 +610,27 @@ MySQL uses &DBD::mysql::FIELD_TYPE_SHORT, &DBD::mysql::FIELD_TYPE_STRING etc.
 =back
 
 
+=head1 SQL EXTENSIONS
+
+Certain metadata functions of mSQL and mysql that are available on the
+C API level, haven't been implemented here. Instead they are implemented
+as "SQL extensions" because they return in fact nothing else but the
+equivalent of a statement handle. These are:
+
+=over 4
+
+=item LISTFIELDS $table
+
+Returns a statement handle that describes the columns of $table.
+Ses the docs of msqlListFields or mysql_list_fields for details.
+
+=item LISTINDEX $table $index
+
+mSQL only; returns a statement handle that describes the index $index
+of table $table. See the docs of msqlListIndex for details.
+
+=back
+
 =head1 COMPATIBILITY ALERT
 
 Certain attributes methods have been declared obsolete or deprecated,
@@ -664,7 +663,8 @@ deprecated, use
 
 deprecated, use
 
-    $dbh->func($table, "listfields")
+    $sth = $dbh->prepare("LISTFIELDS $table")
+    $sth->execute;
 
 =item _ListSelectedFields
 
